/**
 * Copyright (c) 2022
 *    Marcus Britanicus (https://gitlab.com/marcusbritanicus)
 *    Abrar (https://gitlab.com/s96Abrar)
 *    rahmanshaber (https://gitlab.com/rahmanshaber)
 *
 * The DFL::Settings expands the QSettings class.
 *
 * IMPL Class
 *  This class will read the various settings files
 *  The class structure and working is inspired by QSettings.
 *  This is purely an implementational detail.
 *  This class can be modified/removed at anytime.
 **/

#pragma once

#include <QMap>
#include <QList>
#include <QString>
#include <QVariant>
#include <QSettings>

typedef struct changedsetting_t {
    /** Key whose value was changed */
    QString  key;

    /** Value corresponding to the key */
    QVariant value;

    /** What was the change: Addition (a), Deletion (d), Modification (m) */
    char     state;
} ChangedSetting;

typedef QMap<QString, QVariant>   SettingsMap;
typedef QList<ChangedSetting>     ChangedSettings;

namespace DFL {
    class Settings;
    class SettingsImpl;
}

class DFL::SettingsImpl {
    public:

        /**
         * @project -> Project like DesQ, Paper or Oyster
         * @app -> Settings of a specific app: Like Files, Term, Shell, etc...
         * @ConfigPath -> Path where the global config might be strored.
         */
        SettingsImpl( QString project, QString app, QString ConfigPath = "/usr/share" );

        /** To get and set the values for a key */
        QVariant value( const QString& key );
        void setValue( const QString& key, QVariant value );

        /** Keys */
        QStringList allKeys();
        bool hasKey( QString );

        /**
         * Get the changed settings as a list
         * Read the current settings, then compare with the settings stored in @map.
         * Store the changed settings, and update @map.
         * Finally return the changed settings.
         */
        ChangedSettings changedSettings();

        /**
         * @usrSett -> User level settings from ~/.config/<project>/<app>.conf
         * @sysSett -> Distro settings, typically, /etc/xdg/<project>/<app>.conf
         * @defSett -> Default settings, shipped by the devs.
         */
        QSettings *usrSett, *sysSett, *defSett;

    private:
        /** This is the final settings map of the app */
        SettingsMap map;

        /** Load the current settings. Responsible for maintaining usr/sys/def hierarchy */
        SettingsMap readCurrentSettings();
};
